// src/index.ts
import { extname } from "path";
import {
  getCachedDocumentNodeFromSchema
} from "@graphql-codegen/plugin-helpers";
import { schemaPrepend } from "amplience-graphql-codegen-common";
import { snakeCase as snakeCase2 } from "change-case";
import { visit } from "graphql";
import { map, TerraformGenerator as TerraformGenerator2 } from "terraform-generator";

// src/lib/visitor.ts
import {
  maybeDirective,
  maybeDirectiveValue,
  typeUri,
  hasProperty
} from "amplience-graphql-codegen-common";
import { snakeCase, paramCase, capitalCase } from "change-case";
import { fn, arg } from "terraform-generator";
var createObjectTypeVisitor = ({
  tfg,
  hostname,
  visualization,
  contentRepositories,
  slotRepositories,
  contentRepositoriesForEach,
  slotRepositoriesForEach,
  schemaSuffix
}) => (node) => {
  var _a2, _b, _c, _d, _e;
  const directive = maybeDirective(node, "amplienceContentType");
  if (!directive)
    return null;
  const name = snakeCase(node.name.value);
  const isSlot = ((_a2 = maybeDirectiveValue(directive, "kind")) == null ? void 0 : _a2.value) === "SLOT";
  const isAutoSync = ((_b = maybeDirectiveValue(directive, "autoSync")) == null ? void 0 : _b.value) ?? true;
  const schema = tfg.resource("amplience_content_type_schema", name, {
    body: fn(
      "file",
      `\${path.module}/schemas/${paramCase(node.name.value)}${schemaSuffix ? "-" + schemaSuffix : ""}.json`
    ),
    schema_id: typeUri(node, hostname),
    validation_level: isSlot ? "SLOT" : "CONTENT_TYPE",
    auto_sync: isAutoSync
  });
  const shouldVisualize = (_c = maybeDirectiveValue(
    directive,
    "visualizations"
  )) == null ? void 0 : _c.value;
  const iconUrl = directive ? (_d = maybeDirectiveValue(directive, "icon")) == null ? void 0 : _d.value : void 0;
  const dynamicVisualization = visualization == null ? void 0 : visualization.find(hasProperty("for_each"));
  const contentType = tfg.resource("amplience_content_type", name, {
    content_type_uri: schema.attr("schema_id"),
    label: capitalCase(node.name.value),
    icon: iconUrl ? { size: 256, url: iconUrl } : void 0,
    status: "ACTIVE",
    'dynamic"visualization"': shouldVisualize && dynamicVisualization ? dynamicVisualizationBlock(dynamicVisualization) : void 0,
    visualization: shouldVisualize && visualization ? visualization.filter((v) => !v.for_each) : void 0
  });
  const repositoryName = (_e = maybeDirectiveValue(
    directive,
    "repository"
  )) == null ? void 0 : _e.value;
  if ((contentRepositories == null ? void 0 : contentRepositories.length) && !isSlot) {
    tfg.resource("amplience_content_type_assignment", name, {
      content_type_id: contentType.id,
      repository_id: (contentRepositories.find((r) => r.name === repositoryName) ?? contentRepositories[0]).id
    });
  }
  if ((slotRepositories == null ? void 0 : slotRepositories.length) && isSlot) {
    tfg.resource("amplience_content_type_assignment", name, {
      content_type_id: contentType.id,
      repository_id: (slotRepositories.find((r) => r.name === repositoryName) ?? slotRepositories[0]).id
    });
  }
  if (contentRepositoriesForEach && !isSlot) {
    tfg.resource("amplience_content_type_assignment", name, {
      for_each: arg(contentRepositoriesForEach),
      content_type_id: contentType.id,
      repository_id: arg("each.value")
    });
  }
  if (slotRepositoriesForEach && isSlot) {
    tfg.resource("amplience_content_type_assignment", name, {
      for_each: arg(slotRepositoriesForEach),
      content_type_id: contentType.id,
      repository_id: arg("each.value")
    });
  }
  return null;
};
var maybeArg = (value, ...prefixes) => ["var.", "local.", ...prefixes].some((prefix) => value.startsWith(prefix)) ? arg(value) : value;
var dynamicVisualizationBlock = (visualization) => ({
  for_each: arg(visualization.for_each),
  content: {
    label: maybeArg(visualization.label, "visualization"),
    templated_uri: maybeArg(visualization.templated_uri, "visualization"),
    default: visualization.default
  }
});

// src/index.ts
var _a;
var addToSchema = (_a = schemaPrepend.loc) == null ? void 0 : _a.source.body;
var plugin = (schema, _documents, {
  hostname = "https://schema-examples.com",
  visualization,
  content_repositories,
  slot_repositories,
  schemaSuffix,
  add_required_provider = true
}) => {
  const astNode = getCachedDocumentNodeFromSchema(schema);
  const tfg = new TerraformGenerator2(
    add_required_provider ? {
      required_providers: { amplience: map({ source: "labd/amplience" }) }
    } : void 0
  );
  const contentRepositories = content_repositories ? Object.entries(content_repositories).filter(([name]) => name !== "for_each").map(
    ([name, value]) => tfg.data("amplience_content_repository", snakeCase2(name), {
      id: maybeArg(value)
    })
  ) : void 0;
  const slotRepositories = slot_repositories ? Object.entries(slot_repositories).filter(([name]) => name !== "for_each").map(
    ([name, value]) => tfg.data("amplience_content_repository", snakeCase2(name), {
      id: maybeArg(value)
    })
  ) : void 0;
  const slotRepositoriesForEach = slot_repositories == null ? void 0 : slot_repositories.for_each;
  const contentRepositoriesForEach = content_repositories == null ? void 0 : content_repositories.for_each;
  visit(astNode, {
    ObjectTypeDefinition: {
      leave: createObjectTypeVisitor({
        tfg,
        contentRepositories,
        slotRepositories,
        contentRepositoriesForEach,
        slotRepositoriesForEach,
        hostname,
        visualization,
        schemaSuffix
      })
    }
  });
  return tfg.generate().tf;
};
var validate = async (_schema, _documents, config, outputFile) => {
  if (extname(outputFile) !== ".tf") {
    throw new Error(
      `Plugin "amplience-terraform" requires output extension to be ".tf"!`
    );
  }
  if (config.visualization) {
    if (config.visualization.filter((v) => v.for_each).length > 1) {
      throw new Error(
        `You can only have 1 item with a for_each property in your visualization list.`
      );
    }
    if (config.visualization.filter((v) => v.default).length > 1 || config.visualization.some((v) => v.default && v.for_each)) {
      throw new Error(
        `You can only set 1 item, which may not be a for_each-item to be the default.`
      );
    }
  }
  if (config.content_repositories) {
    validateRepositoryConfig(config.content_repositories);
  }
  if (config.slot_repositories) {
    validateRepositoryConfig(config.slot_repositories);
  }
};
function validateRepositoryConfig(repositoryMap) {
  const repositoryKeys = Object.keys(repositoryMap);
  if (repositoryKeys.length > 1 && repositoryMap.for_each) {
    throw new Error(
      `for_each should not be used when multiple repositories are defined to prevent duplicate resources.`
    );
  }
}
export {
  addToSchema,
  plugin,
  validate
};
//# sourceMappingURL=index.mjs.map